@@ -12,14 +12,14 @@ Python libraries, though it does make heavy use of external packaging commands.
 Visit <https://github.com/datalad/datalad-installer> for more information.
 """
 
-__version__ = "0.5.3"
+__version__ = "0.5.5"
 __author__ = "The DataLad Team and Contributors"
 __author_email__ = "team@datalad.org"
 __license__ = "MIT"
 __url__ = "https://github.com/datalad/datalad-installer"
 
 from abc import ABC, abstractmethod
-from contextlib import contextmanager
+from contextlib import contextmanager, suppress
 import ctypes
 from enum import Enum
 from functools import total_ordering
@@ -38,6 +38,7 @@ import subprocess
 import sys
 import tempfile
 import textwrap
+from time import sleep
 from typing import (
     Any,
     Callable,
@@ -98,7 +99,7 @@ class Immediate:
 
 
 class VersionRequest(Immediate):
-    """ `Immediate` representing a ``--version`` option """
+    """`Immediate` representing a ``--version`` option"""
 
     def __eq__(self, other: Any) -> bool:
         if type(self) is type(other):
@@ -108,7 +109,7 @@ class VersionRequest(Immediate):
 
 
 class HelpRequest(Immediate):
-    """ `Immediate` representing a ``--help`` option """
+    """`Immediate` representing a ``--help`` option"""
 
     def __init__(self, component: Optional[str]) -> None:
         #: The component for which help was requested, or `None` if the
@@ -200,7 +201,7 @@ class Option:
 
     @property
     def option_name(self) -> str:
-        """ Display name for the option """
+        """Display name for the option"""
         if self.longopts:
             return f"--{self.longopts[0]}"
         else:
@@ -383,7 +384,7 @@ class OptionParser:
 
 
 class UsageError(Exception):
-    """ Raised when an error occurs while processing command-line options """
+    """Raised when an error occurs while processing command-line options"""
 
     def __init__(self, message: str, component: Optional[str] = None) -> None:
         #: The error message
@@ -407,7 +408,7 @@ class ParsedArgs(NamedTuple):
 
 
 class ComponentRequest:
-    """ A request for a component parsed from command-line arguments """
+    """A request for a component parsed from command-line arguments"""
 
     def __init__(self, name: str, **kwargs: Any) -> None:
         self.name: str = name
@@ -430,7 +431,7 @@ class ComponentRequest:
 
 
 class CondaInstance(NamedTuple):
-    """ A Conda installation or environment """
+    """A Conda installation or environment"""
 
     #: The root of the Conda installation
     basepath: Path
@@ -440,7 +441,7 @@ class CondaInstance(NamedTuple):
 
     @property
     def conda_exe(self) -> Path:
-        """ The path to the Conda executable """
+        """The path to the Conda executable"""
         if ON_WINDOWS:
             return self.basepath / "Scripts" / "conda.exe"
         else:
@@ -464,7 +465,7 @@ CommandList = List[Tuple[str, Path]]
 
 
 class DataladInstaller:
-    """ The script's primary class, a manager & runner of components """
+    """The script's primary class, a manager & runner of components"""
 
     COMPONENTS: ClassVar[Dict[str, Type["Component"]]] = {}
 
@@ -540,7 +541,7 @@ class DataladInstaller:
     def register_component(
         cls, name: str
     ) -> Callable[[Type["Component"]], Type["Component"]]:
-        """ A decorator for registering concrete `Component` subclasses """
+        """A decorator for registering concrete `Component` subclasses"""
 
         def decorator(component: Type["Component"]) -> Type["Component"]:
             cls.COMPONENTS[name] = component
@@ -551,14 +552,14 @@ class DataladInstaller:
     def __enter__(self) -> "DataladInstaller":
         return self
 
-    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None:
+    def __exit__(self, exc_type: Any, _exc_value: Any, _exc_tb: Any) -> None:
         if exc_type is None:
             # Ensure env write files at least exist
             for p in self.env_write_files:
                 p.touch()
 
     def ensure_env_write_file(self) -> None:
-        """ If there are no env write files registered, add one """
+        """If there are no env write files registered, add one"""
         if not self.env_write_files:
             fd, fpath = tempfile.mkstemp(prefix="dl-env-", suffix=".sh")
             os.close(fd)
@@ -589,6 +590,16 @@ class DataladInstaller:
                     self.sudo_confirm = SudoConfirm.OK
             runcmd("sudo", *args, **kwargs)
 
+    def run_maybe_elevated(self, *args: Any, **kwargs: Any) -> None:
+        try:
+            runcmd(*args, **kwargs)
+        except OSError as e:
+            if e.winerror == 740:  # type: ignore[attr-defined]
+                log.info("Operation requires elevation; rerunning as administrator")
+                self.sudo(*args, **kwargs)
+            else:
+                raise
+
     @classmethod
     def parse_args(cls, args: List[str]) -> Union[Immediate, ParsedArgs]:
         """
@@ -693,7 +704,7 @@ class DataladInstaller:
         return 0 if ok else 1
 
     def addenv(self, line: str) -> None:
-        """ Write a line to the env write files """
+        """Write a line to the env write files"""
         log.debug("Adding line %r to env_write_files", line)
         for p in self.env_write_files:
             with p.open("a") as fp:
@@ -712,7 +723,7 @@ class DataladInstaller:
         self.addenv(line)
 
     def addcomponent(self, name: str, **kwargs: Any) -> None:
-        """ Provision the given component """
+        """Provision the given component"""
         try:
             component = self.COMPONENTS[name]
         except AttributeError:
@@ -781,7 +792,7 @@ class Component(ABC):
 
 @DataladInstaller.register_component("venv")
 class VenvComponent(Component):
-    """ Creates a Python virtual environment using ``python -m venv`` """
+    """Creates a Python virtual environment using ``python -m venv``"""
 
     OPTION_PARSER = OptionParser(
         "venv",
@@ -800,6 +811,12 @@ class VenvComponent(Component):
                 converter=shlex.split,
                 help="Extra arguments to pass to the venv command",
             ),
+            # For use in testing against the dev version of pip:
+            Option(
+                "--dev-pip",
+                is_flag=True,
+                help="Install the development version of pip from GitHub",
+            ),
         ],
     )
 
@@ -807,6 +824,7 @@ class VenvComponent(Component):
         self,
         path: Optional[Path] = None,
         extra_args: Optional[List[str]] = None,
+        dev_pip: bool = False,
         **kwargs: Any,
     ) -> None:
         log.info("Creating a virtual environment")
@@ -822,12 +840,24 @@ class VenvComponent(Component):
             cmd.extend(extra_args)
         cmd.append(str(path))
         runcmd(*cmd)
-        self.manager.installer_stack.append(PipInstaller(self.manager, path))
+        installer = PipInstaller(self.manager, path)
+        if dev_pip:
+            runcmd(
+                installer.python,
+                "-m",
+                "pip",
+                "install",
+                "pip @ git+https://github.com/pypa/pip",
+            )
+        else:
+            # Ensure we have a recent pip
+            runcmd(installer.python, "-m", "pip", "install", "--upgrade", "pip")
+        self.manager.installer_stack.append(installer)
 
 
 @DataladInstaller.register_component("miniconda")
 class MinicondaComponent(Component):
-    """ Installs Miniconda """
+    """Installs Miniconda"""
 
     OPTION_PARSER = OptionParser(
         "miniconda",
@@ -938,7 +968,7 @@ class MinicondaComponent(Component):
 
 @DataladInstaller.register_component("conda-env")
 class CondaEnvComponent(Component):
-    """ Creates a Conda environment """
+    """Creates a Conda environment"""
 
     OPTION_PARSER = OptionParser(
         "conda-env",
@@ -1000,7 +1030,7 @@ class CondaEnvComponent(Component):
 
 @DataladInstaller.register_component("neurodebian")
 class NeurodebianComponent(Component):
-    """ Installs & configures NeuroDebian """
+    """Installs & configures NeuroDebian"""
 
     OPTION_PARSER = OptionParser(
         "neurodebian",
@@ -1088,7 +1118,7 @@ class InstallableComponent(Component):
 
     @classmethod
     def register_installer(cls, installer: Type["Installer"]) -> Type["Installer"]:
-        """ A decorator for registering concrete `Installer` subclasses """
+        """A decorator for registering concrete `Installer` subclasses"""
         cls.INSTALLERS[installer.NAME] = installer
         methods = cls.OPTION_PARSER.options["--method"].choices
         assert methods is not None
@@ -1098,7 +1128,7 @@ class InstallableComponent(Component):
         return installer
 
     def get_installer(self, name: str) -> "Installer":
-        """ Retrieve & instantiate the installer with the given name """
+        """Retrieve & instantiate the installer with the given name"""
         try:
             installer_cls = self.INSTALLERS[name]
         except KeyError:
@@ -1125,7 +1155,7 @@ class InstallableComponent(Component):
 
 @DataladInstaller.register_component("git-annex")
 class GitAnnexComponent(InstallableComponent):
-    """ Installs git-annex """
+    """Installs git-annex"""
 
     NAME = "git-annex"
 
@@ -1146,7 +1176,7 @@ class GitAnnexComponent(InstallableComponent):
 
 @DataladInstaller.register_component("datalad")
 class DataladComponent(InstallableComponent):
-    """ Installs Datalad """
+    """Installs Datalad"""
 
     NAME = "datalad"
 
@@ -1166,7 +1196,7 @@ class DataladComponent(InstallableComponent):
 
 
 class Installer(ABC):
-    """ An abstract base class for installation methods for packages """
+    """An abstract base class for installation methods for packages"""
 
     NAME: ClassVar[str]
 
@@ -1230,7 +1260,7 @@ EXTRA_ARGS_OPTION = Option(
 @GitAnnexComponent.register_installer
 @DataladComponent.register_installer
 class AptInstaller(Installer):
-    """ Installs via apt-get """
+    """Installs via apt-get"""
 
     NAME = "apt"
 
@@ -1283,7 +1313,7 @@ class AptInstaller(Installer):
 @DataladComponent.register_installer
 @GitAnnexComponent.register_installer
 class HomebrewInstaller(Installer):
-    """ Installs via brew (Homebrew) """
+    """Installs via brew (Homebrew)"""
 
     NAME = "brew"
 
@@ -1436,7 +1466,7 @@ class PipInstaller(Installer):
 
 @GitAnnexComponent.register_installer
 class NeurodebianInstaller(AptInstaller):
-    """ Installs via apt-get and the NeuroDebian repositories """
+    """Installs via apt-get and the NeuroDebian repositories"""
 
     NAME = "neurodebian"
 
@@ -1453,7 +1483,7 @@ class NeurodebianInstaller(AptInstaller):
 @GitAnnexComponent.register_installer
 @DataladComponent.register_installer
 class DebURLInstaller(Installer):
-    """ Installs a ``*.deb`` package by URL """
+    """Installs a ``*.deb`` package by URL"""
 
     NAME = "deb-url"
 
@@ -1554,7 +1584,7 @@ class AutobuildSnapshotInstaller(Installer):
 
 @GitAnnexComponent.register_installer
 class AutobuildInstaller(AutobuildSnapshotInstaller):
-    """ Installs the latest official build of git-annex from kitenet.net """
+    """Installs the latest official build of git-annex from kitenet.net"""
 
     NAME = "autobuild"
 
@@ -1599,7 +1629,7 @@ class SnapshotInstaller(AutobuildSnapshotInstaller):
 @GitAnnexComponent.register_installer
 @DataladComponent.register_installer
 class CondaInstaller(Installer):
-    """ Installs via conda """
+    """Installs via conda"""
 
     NAME = "conda"
 
@@ -1650,7 +1680,22 @@ class CondaInstaller(Installer):
             cmd.append(package)
         else:
             cmd.append(f"{package}={version}")
-        runcmd(*cmd)
+        i = 0
+        while True:
+            try:
+                runcmd(*cmd)
+            except subprocess.CalledProcessError as e:
+                if i < 3:
+                    log.error(
+                        "Command failed with exit status %d; sleeping and retrying",
+                        e.returncode,
+                    )
+                    i += 1
+                    sleep(5)
+                else:
+                    raise
+            else:
+                break
         binpath = conda.bindir
         log.debug("Installed program directory: %s", binpath)
         return binpath
@@ -1694,11 +1739,16 @@ class DataladGitAnnexBuildInstaller(Installer):
         if kwargs:
             log.warning("Ignoring extra installer arguments: %r", kwargs)
         assert package == "git-annex"
-        with tempfile.TemporaryDirectory() as tmpdir_:
+        # Try to ignore cleanup errors on Windows:
+        with suppress(NotADirectoryError), tempfile.TemporaryDirectory() as tmpdir_:
             tmpdir = Path(tmpdir_)
             if ON_LINUX:
                 self.download("ubuntu", tmpdir)
                 (debpath,) = tmpdir.glob("*.deb")
+                if install_dir is None and deb_pkg_installed("git-annex"):
+                    self.manager.sudo(
+                        "dpkg", "--remove", "--ignore-depends=git-annex", "git-annex"
+                    )
                 binpath = install_deb(
                     debpath,
                     self.manager,
@@ -1712,7 +1762,7 @@ class DataladGitAnnexBuildInstaller(Installer):
             elif ON_WINDOWS:
                 self.download("windows", tmpdir)
                 (exepath,) = tmpdir.glob("*.exe")
-                runcmd(exepath, "/S")
+                self.manager.run_maybe_elevated(exepath, "/S")
                 binpath = Path("C:/Program Files", "Git", "usr", "bin")
                 self.manager.addpath(binpath)
             else:
@@ -1914,14 +1964,7 @@ class DataladPackagesBuildInstaller(Installer):
                 f"https://datasets.datalad.org/datalad/packages/windows/{exefile}",
                 exepath,
             )
-            try:
-                runcmd(exepath, "/S")
-            except OSError as e:
-                if e.winerror == 740:  # type: ignore[attr-defined]
-                    log.info("Operation requires elevation; rerunning as administrator")
-                    self.manager.sudo(exepath, "/S")
-                else:
-                    raise
+            self.manager.run_maybe_elevated(exepath, "/S")
             binpath = Path("C:/Program Files", "Git", "usr", "bin")
             self.manager.addpath(binpath)
         else:
@@ -1936,7 +1979,7 @@ class DataladPackagesBuildInstaller(Installer):
 
 @GitAnnexComponent.register_installer
 class DMGInstaller(Installer):
-    """ Installs a local ``*.dmg`` file """
+    """Installs a local ``*.dmg`` file"""
 
     NAME = "dmg"
 
@@ -2005,7 +2048,7 @@ def compose_pip_requirement(
     urlspec: Optional[str] = None,
     extras: Optional[str] = None,
 ) -> str:
-    """ Compose a PEP 503 requirement specifier """
+    """Compose a PEP 503 requirement specifier"""
     req = package
     if extras is not None:
         req += f"[{extras}]"
@@ -2020,19 +2063,19 @@ def compose_pip_requirement(
 
 
 def mktempdir(prefix: str) -> Path:
-    """ Create a directory in ``$TMPDIR`` with the given prefix """
+    """Create a directory in ``$TMPDIR`` with the given prefix"""
     return Path(tempfile.mkdtemp(prefix=prefix))
 
 
 def runcmd(*args: Any, **kwargs: Any) -> subprocess.CompletedProcess:
-    """ Run (and log) a given command.  Raise an error if it fails. """
+    """Run (and log) a given command.  Raise an error if it fails."""
     arglist = [str(a) for a in args]
     log.info("Running: %s", " ".join(map(shlex.quote, arglist)))
     return subprocess.run(arglist, check=True, **kwargs)
 
 
-def readcmd(*args: Any, **kwargs: Any) -> str:
-    """ Run a command, capturing & returning its stdout """
+def readcmd(*args: Any) -> str:
+    """Run a command, capturing & returning its stdout"""
     s = runcmd(*args, stdout=subprocess.PIPE, universal_newlines=True).stdout
     assert isinstance(s, str)
     return s
@@ -2041,7 +2084,7 @@ def readcmd(*args: Any, **kwargs: Any) -> str:
 def install_git_annex_dmg(
     dmgpath: Union[str, os.PathLike], manager: DataladInstaller
 ) -> Path:
-    """ Install git-annex from a DMG file at ``dmgpath`` """
+    """Install git-annex from a DMG file at ``dmgpath``"""
     runcmd("hdiutil", "attach", dmgpath)
     runcmd("rsync", "-a", "/Volumes/git-annex/git-annex.app", "/Applications/")
     runcmd("hdiutil", "detach", "/Volumes/git-annex/")
@@ -2057,20 +2100,26 @@ def install_deb(
     install_dir: Optional[Path] = None,
     extra_args: Optional[List[str]] = None,
 ) -> Path:
-    cmd: List[Union[str, os.PathLike]] = ["dpkg"]
-    if extra_args is not None:
-        cmd.extend(extra_args)
     if install_dir is None:
+        cmd: List[Union[str, os.PathLike]] = ["dpkg"]
+        if extra_args is not None:
+            cmd.extend(extra_args)
         cmd.append("-i")
         cmd.append(debpath)
         manager.sudo(*cmd)
         return Path("/usr/bin")
     else:
+        if extra_args:
+            log.warning("Not using dpkg; ignoring --extra-args")
+        assert os.path.isabs(debpath)
         install_dir.mkdir(parents=True, exist_ok=True)
-        cmd.append("-x")
-        cmd.append(debpath)
-        cmd.append(install_dir)
-        runcmd(*cmd)
+        install_dir = install_dir.resolve()
+        with tempfile.TemporaryDirectory() as tmpdir:
+            oldpwd = os.getcwd()
+            os.chdir(tmpdir)
+            runcmd("ar", "-x", debpath)
+            runcmd("tar", "-C", install_dir, "-xzf", "data.tar.gz")
+            os.chdir(oldpwd)
         manager.addpath(install_dir / bin_path)
         return install_dir / bin_path
 
@@ -2134,6 +2183,16 @@ def untmppath(path: Path) -> Path:
         return path
 
 
+def deb_pkg_installed(package: str) -> bool:
+    r = subprocess.run(
+        ["dpkg-query", "-Wf", "${db:Status-Status}", package],
+        stdout=subprocess.PIPE,
+        stderr=subprocess.DEVNULL,
+        universal_newlines=True,
+    )
+    return r.returncode == 0 and r.stdout == "installed"
+
+
 def main(argv: Optional[List[str]] = None) -> int:
     with DataladInstaller() as manager:
         return manager.main(argv)
