@@ -4,6 +4,7 @@ from pathlib import Path
 import shlex
 import shutil
 import subprocess
+import sys
 import tempfile
 import pytest
 import datalad_installer
@@ -225,6 +226,26 @@ def test_install_venv_datalad(tmp_path):
     assert (venv_path / bin_path("datalad")).exists()
 
 
+@pytest.mark.skipif(
+    sys.version_info[:2] < (3, 7), reason="dev pip no longer supports Python < 3.7"
+)
+def test_install_venv_dev_pip_datalad(tmp_path):
+    venv_path = tmp_path / "venv"
+    r = main(
+        [
+            "datalad_installer.py",
+            "venv",
+            "--path",
+            str(venv_path),
+            "--dev-pip",
+            "datalad",
+        ]
+    )
+    assert r == 0
+    assert (venv_path / bin_path("python")).exists()
+    assert (venv_path / bin_path("datalad")).exists()
+
+
 @pytest.mark.miniconda
 def test_install_miniconda_conda_env_venv_datalad(tmp_path):
     venv_path = tmp_path / "venv"
